-- date: 08.01.2017
-- author: Raphael Greshake
-- copyright (c): Creative Mesh UG (haftungsbeschrnkt)

-- loads fillTypes, particles, effects, and adds fillTypes to tipTriggers/fillPoints

MaterialLoader = {}
MaterialLoader.modDir = g_currentModDirectory;
MaterialLoader.strawPelletsPrice = 0.6;
MaterialLoader.hayPelletsPrice = 0.8;
MaterialLoader.molassesPrice = 3;

-- register strawPellets
local category = 1; -- bulk
local showOnPriceTable = true;
local massPerLiter = 0.0005;
local maxPhysicalSurfaceAngle = 50; --math.rad(25);
local hudOverlayFilename = g_currentModDirectory.."hud/strawPellets_icon.dds";
local hudOverlayFilenameSmall = g_currentModDirectory.."hud/strawPellets_icon_small.dds";
FillUtil.registerFillType("strawPellets", g_i18n:getText("fillType_strawPellets"), category, MaterialLoader.strawPelletsPrice, showOnPriceTable, hudOverlayFilename, hudOverlayFilenameSmall, massPerLiter, maxPhysicalSurfaceAngle);

-- register hayPellets
local category = 1; -- bulk
local showOnPriceTable = true;
local massPerLiter = 0.0005;
local maxPhysicalSurfaceAngle = 50; --math.rad(25);
local hudOverlayFilename = g_currentModDirectory.."hud/hayPellets_icon.dds";
local hudOverlayFilenameSmall = g_currentModDirectory.."hud/hayPellets_icon_small.dds";
FillUtil.registerFillType("hayPellets", g_i18n:getText("fillType_hayPellets"), category, MaterialLoader.hayPelletsPrice, showOnPriceTable, hudOverlayFilename, hudOverlayFilenameSmall, massPerLiter, maxPhysicalSurfaceAngle);

-- register strawPellets heap
local strawPellets = FillUtil.fillTypeNameToInt["strawPellets"];
local collisionScale = 1;
local collisionBaseOffset = 0.2;
local minCollisionOffset = 0.2;
local maxCollisionOffset = 0.5;
local fillToGroundScale = 1;
local allowsSmoothing = false;
local diffuseMapFilename = MaterialLoader.modDir.."shared/planes/heap_strawPellets_diffuse.dds";
local normalMapFilename = MaterialLoader.modDir.."shared/planes/heap_strawPellets_normal.dds";
local distanceFilename = MaterialLoader.modDir.."shared/planes/heap_strawPellets_distance.dds";
TipUtil.registerDensityMapHeightType(strawPellets, maxPhysicalSurfaceAngle, collisionScale, collisionBaseOffset, minCollisionOffset, maxCollisionOffset, fillToGroundScale, allowsSmoothing, diffuseMapFilename, normalMapFilename, distanceFilename);
	
-- register hayPellets heap
local hayPellets = FillUtil.fillTypeNameToInt["hayPellets"];
local collisionScale = 1;
local collisionBaseOffset = 0.2;
local minCollisionOffset = 0.2;
local maxCollisionOffset = 0.5;
local fillToGroundScale = 1;
local allowsSmoothing = false;
local diffuseMapFilename = MaterialLoader.modDir.."shared/planes/heap_hayPellets_diffuse.dds";
local normalMapFilename = MaterialLoader.modDir.."shared/planes/heap_strawPellets_normal.dds";
local distanceFilename = MaterialLoader.modDir.."shared/planes/heap_strawPellets_distance.dds";
TipUtil.registerDensityMapHeightType(hayPellets, maxPhysicalSurfaceAngle, collisionScale, collisionBaseOffset, minCollisionOffset, maxCollisionOffset, fillToGroundScale, allowsSmoothing, diffuseMapFilename, normalMapFilename, distanceFilename);
		
-- register pellets as food
FillUtil.registerFillTypeInFoodGroup(2, 1, hayPellets);
FillUtil.registerFillTypeInFoodGroup(3, 2, hayPellets);
FillUtil.registerFillTypeInFoodGroup(1, 2, hayPellets);
-- cow: 1, sheep: 2, pic: 3

-- add straw to animal husbandry
local oldFeedingTroughTipTriggerGetAllowFillTypeFromTool = FeedingTroughTipTrigger.getAllowFillTypeFromTool;
FeedingTroughTipTrigger.getAllowFillTypeFromTool = function(self, fillType, toolType)
	if fillType == FillUtil.FILLTYPE_STRAWPELLETS then
		return oldFeedingTroughTipTriggerGetAllowFillTypeFromTool(self, FillUtil.FILLTYPE_STRAW, toolType);
	else
		return oldFeedingTroughTipTriggerGetAllowFillTypeFromTool(self, fillType, toolType);
	end;
end;

local oldFeedingTroughTipTriggerAddFillLevelFromTool = FeedingTroughTipTrigger.addFillLevelFromTool;
FeedingTroughTipTrigger.addFillLevelFromTool = function(self, trailerOrShovel, fillDelta, fillType, toolType)
	if fillType == FillUtil.FILLTYPE_STRAWPELLETS then
		return oldFeedingTroughTipTriggerAddFillLevelFromTool(self, trailerOrShovel, fillDelta*3, FillUtil.FILLTYPE_STRAW, toolType)/3;
	else
		return oldFeedingTroughTipTriggerAddFillLevelFromTool(self, trailerOrShovel, fillDelta, fillType, toolType);
	end;
end;

function MaterialLoader:loadMap()
	if not self.gameIsOn then
		MaterialUtil.registerParticleType("sievedrum");
		MaterialUtil.registerParticleType("UNLOADING_NOGRAVITY");
		MaterialUtil.registerMaterialType("rarelycoveredbelt");
		
		function loadMaterialHolder(nodeId)
			-- plane and icon
			MaterialUtil.onCreateMaterial(_, Utils.indexToObject(nodeId, "0|0"));
			MaterialUtil.onCreateMaterial(_, Utils.indexToObject(nodeId, "0|1"));
		
			-- unloading 
			MaterialUtil.onCreateMaterial(_, Utils.indexToObject(nodeId, "1|0"));
			MaterialUtil.onCreateMaterial(_, Utils.indexToObject(nodeId, "1|1"));
			MaterialUtil.onCreateMaterial(_, Utils.indexToObject(nodeId, "1|2"));
			
			-- pipe effect
			MaterialUtil.onCreateMaterial(_, Utils.indexToObject(nodeId, "2|0"));	
			
			-- belt effect
			MaterialUtil.onCreateMaterial(_, Utils.indexToObject(nodeId, "2|1"));
			
			-- leveler effect
			MaterialUtil.onCreateMaterial(_, Utils.indexToObject(nodeId, "2|2"));
			
			-- smoke effect
			MaterialUtil.onCreateMaterial(_, Utils.indexToObject(nodeId, "2|3"));
			
			-- rarely covered belt
			MaterialUtil.onCreateMaterial(_, Utils.indexToObject(nodeId, "2|4"));
		end;
		
		function loadParticles(nodeId)
			-- sieveDrum PS 
			MaterialUtil.onCreateParticleSystem(_, Utils.indexToObject(nodeId, "0|0"));
			-- unloading ps
			MaterialUtil.onCreateParticleSystem(_, Utils.indexToObject(nodeId, "0|0"));
			-- unload ps no gravity for sievedrum
			MaterialUtil.onCreateParticleSystem(_, Utils.indexToObject(nodeId, "0|0"));		
		end;

		self.materialHolderStrawPellets = loadI3DFile(MaterialLoader.modDir.."shared/planes/strawPellets_Holder.i3d", false, false, false);
		loadMaterialHolder(self.materialHolderStrawPellets);
		
		self.materialHolderHayPellets = loadI3DFile(MaterialLoader.modDir.."shared/planes/hayPellets_Holder.i3d", false, false, false);
		loadMaterialHolder(self.materialHolderHayPellets);
		
		self.strawPelletsParticlesRoot = loadI3DFile(MaterialLoader.modDir.."shared/planes/strawPellets_particles.i3d", false, false, false);
		loadParticles(self.strawPelletsParticlesRoot);
		--delete(strawPelletsParticlesRoot);
		
		self.hayPelletsParticlesRoot = loadI3DFile(MaterialLoader.modDir.."shared/planes/hayPellets_particles.i3d", false, false, false);
		loadParticles(self.hayPelletsParticlesRoot);
		--delete(hayPelletsParticlesRoot);
		
		function addFillTypeToSellTrigger(tipTrigger, fillType, conditionFillType, price)
			if tipTrigger.acceptedFillTypes[conditionFillType] ~= nil then
				local allowedToolTypes = {}
				for k, v in pairs(tipTrigger.allowedToolTypes[conditionFillType]) do
					table.insert(allowedToolTypes, k);
				end;
				tipTrigger:addAcceptedFillType(fillType, price, true, false, allowedToolTypes);
			end;
		end;
		
		-- register into tip triggers
		for tipTrigger, v in pairs(g_currentMission.tipTriggers) do
			if tipTrigger.isSellingPoint then
				if tipTrigger.acceptedFillTypes ~= nil and tipTrigger.allowedToolTypes ~= nil and tipTrigger.addAcceptedFillType ~= nil then
					addFillTypeToSellTrigger(tipTrigger, FillUtil.fillTypeNameToInt["strawPellets"], FillUtil.fillTypeNameToInt["woodChips"], MaterialLoader.strawPelletsPrice);
					addFillTypeToSellTrigger(tipTrigger, FillUtil.fillTypeNameToInt["strawPellets"], FillUtil.fillTypeNameToInt["straw"], MaterialLoader.strawPelletsPrice);
					addFillTypeToSellTrigger(tipTrigger, FillUtil.fillTypeNameToInt["hayPellets"], FillUtil.fillTypeNameToInt["dryGrass_windrow"], MaterialLoader.hayPelletsPrice);
					tipTrigger:initPricingDynamics(); -- might be a bit dangerous since the content of this function is not documented
				end;
			end;		
		end;

		self.gameIsOn = true;
	end;
end;

function MaterialLoader:deleteMap()
	if self.gameIsOn then
		delete(self.materialHolderStrawPellets);
		delete(self.materialHolderHayPellets);
		
		delete(self.strawPelletsParticlesRoot);
		delete(self.hayPelletsParticlesRoot);

		self.gameIsOn = false;
	end;
end;

function MaterialLoader:mouseEvent(posX, posY, isDown, isUp, button)
end;

function MaterialLoader:keyEvent(unicode, sym, modifier, isDown)
end;

function MaterialLoader:update(dt)
	
end;

function MaterialLoader:draw()

end;

addModEventListener(MaterialLoader);